local L = LibStub("AceLocale-3.0"):GetLocale("Elephant")
local Elephant = LibStub("AceAddon-3.0"):GetAddon("Elephant")

function Elephant:OnInitialize()
	--[[ Registering database with defaults: cloning objects to avoid problems ]]
	Elephant.db = LibStub("AceDB-3.0"):New("ElephantDB", {
		profile = Elephant:clone(Elephant.defaultConf.savedconfdefaults)
	})
	Elephant.dbpc = LibStub("AceDB-3.0"):New("ElephantDBPerChar", {
		char = Elephant:clone(Elephant.defaultConf.savedpercharconfdefaults)
	})

	local Fonts = {
		['GameFontNormal'] = {
			['Font'] = { GameFontNormal:GetFont() },
			['TextColor'] = { GameFontNormal:GetTextColor() }
		},
		['GameFontNormalSmall'] = {
			['Font'] = { GameFontNormalSmall:GetFont() },
			['TextColor'] = { GameFontNormalSmall:GetTextColor() }
		},
		['ChatFontNormal'] = {
			['Font'] = { ChatFontNormal:GetFont() },
			['TextColor'] = { ChatFontNormal:GetTextColor() }
		}
	}
	
	--[[ Creating gui pointers holder ]]
	Elephant.gui = {}
	
	--[[ Only one getglobal() to copy frame ]]
	Elephant.gui.copyFrame = getglobal("ElephantCopyFrame")
	Elephant.gui.copyFrame:SetBackdrop ( {
		bgFile = "Interface/ChatFrame/ChatFrameBackground",
		edgeFile = [[Interface\AddOns\Elephant\roth]],
		tile = true,
		tileSize = 16,
		edgeSize = 16,
		insets = { left = 5, right = 5, top = 5, bottom = 5 },
	} )
	Elephant.gui.copyFrame:SetBackdropColor(0.0, 0.0, 0.0, 0.45)
	
	--[[ General frame ]]
	Elephant.gui.mainFrame = CreateFrame("Frame", "ElephantFrame", UIParent)
	Elephant.gui.mainFrame:EnableMouse(true)
	Elephant.gui.mainFrame:SetMovable(true)
	Elephant.gui.mainFrame:SetToplevel(true)
	Elephant.gui.mainFrame:SetWidth(600)
	Elephant.gui.mainFrame:SetHeight(580)
	Elephant.gui.mainFrame:SetPoint("TOP", UIParent, "TOP", Elephant.defaultConf.position.x, Elephant.defaultConf.position.y)
	Elephant.gui.mainFrame:SetBackdrop ( {
		bgFile = "Interface/ChatFrame/ChatFrameBackground",
		edgeFile = [[Interface\AddOns\Elephant\roth]],
		tile = true,
		tileSize = 16,
		edgeSize = 16,
		insets = { left = 5, right = 5, top = 5, bottom = 5 },
	} )
	Elephant.gui.mainFrame:SetBackdropColor(0.0, 0.0, 0.0, 0.45)
	Elephant.gui.mainFrame:SetScript("OnShow", function()
		Elephant.gui.copyFrame:Hide()
		PlaySound("SPELLBOOKOPEN")
	end)
	Elephant.gui.mainFrame:SetScript("OnHide", function()
		PlaySound("SPELLBOOKCLOSE")
		CloseDropDownMenus()
	end)
	Elephant.gui.mainFrame:SetScript("OnMouseDown", function(self, button)
		if button == "RightButton" then
			Elephant.gui.mainFrame:StartMoving()
		end
		if button == "MiddleButton" then
			Elephant:ResetPosition()
		end
	end)
	Elephant.gui.mainFrame:SetScript("OnMouseUp", function() Elephant.gui.mainFrame:StopMovingOrSizing() end)
	
	local elephantTitle = Elephant.gui.mainFrame:CreateTexture("ElephantTitle", "ARTWORK")
	elephantTitle:SetTexture("Interface/BUTTONS/UI-DialogBox-Button-Up")
	elephantTitle:SetWidth(128)
	elephantTitle:SetHeight(42)
	elephantTitle:SetPoint("TOP", 0, 12)
	
	local elephantTitleText = Elephant.gui.mainFrame:CreateFontString()
	elephantTitleText:SetFont(Fonts.GameFontNormal.Font[1], Fonts.GameFontNormal.Font[2], "OUTLINE")
	elephantTitleText:SetTextColor(Fonts.GameFontNormal.TextColor[1], Fonts.GameFontNormal.TextColor[2], Fonts.GameFontNormal.TextColor[3], Fonts.GameFontNormal.TextColor[4])
	elephantTitleText:SetPoint("TOP", elephantTitle, "TOP", 0, -8)
	elephantTitleText:SetText("Elephant r" .. tonumber((string.gsub("$Revision: 156 $", "^.-(%d+).-$", "%1"))))
	
	local elphantTitleOverFrame = CreateFrame("Frame", "ElephantTitleOverFrame", Elephant.gui.mainFrame)
	elphantTitleOverFrame:SetWidth(128)
	elphantTitleOverFrame:SetHeight(30)
	elphantTitleOverFrame:SetPoint("TOP", 0, 12)
	elphantTitleOverFrame:SetScript("OnEnter", function()
		Elephant:SetTooltip(elphantTitleOverFrame, L['move'])
	end)
	elphantTitleOverFrame:SetScript("OnLeave", function() Elephant:UnsetTooltip() end)
	elphantTitleOverFrame:EnableMouse(true)
	elphantTitleOverFrame:SetScript("OnMouseDown", function(self, button)
		if button == "RightButton" then
			Elephant.gui.mainFrame:StartMoving()
		end
		if button == "MiddleButton" then
			Elephant:ResetPosition()
		end
	end)
	elphantTitleOverFrame:SetScript("OnMouseUp", function() Elephant.gui.mainFrame:StopMovingOrSizing() end)
	
	local elephantTitleInfoFrame = CreateFrame("Frame", "ElephantTitleInfoFrame", Elephant.gui.mainFrame)
	elephantTitleInfoFrame:SetAllPoints(Elephant.gui.mainFrame)
	
	Elephant.gui.titleInfoMaxLog = elephantTitleInfoFrame:CreateFontString()
	Elephant.gui.titleInfoMaxLog:SetFont(Fonts.GameFontNormalSmall.Font[1], Fonts.GameFontNormalSmall.Font[2])
	Elephant.gui.titleInfoMaxLog:SetTextColor(Elephant.defaultConf.colors.maxlog.r, Elephant.defaultConf.colors.maxlog.g, Elephant.defaultConf.colors.maxlog.b, Elephant.defaultConf.colors.maxlog.a)
	Elephant.gui.titleInfoMaxLog:SetPoint("TOPRIGHT", Elephant.gui.mainFrame, "TOPRIGHT", -60, -15)
	
	Elephant.gui.titleInfoTab = elephantTitleInfoFrame:CreateFontString()
	Elephant.gui.titleInfoTab:SetFont(Fonts.GameFontNormalSmall.Font[1], Fonts.GameFontNormalSmall.Font[2])
	Elephant.gui.titleInfoTab:SetTextColor(Elephant.defaultConf.colors.tab.r, Elephant.defaultConf.colors.tab.g, Elephant.defaultConf.colors.tab.b, Elephant.defaultConf.colors.tab.a)
	Elephant.gui.titleInfoTab:SetPoint("TOPLEFT", Elephant.gui.mainFrame, "TOPLEFT", 35, -15)
	Elephant.gui.titleInfoTab:SetText("< >")
	
	Elephant.gui.titleInfoCurrentLine = elephantTitleInfoFrame:CreateFontString()
	Elephant.gui.titleInfoCurrentLine:SetFont(Fonts.GameFontNormalSmall.Font[1], Fonts.GameFontNormalSmall.Font[2])
	Elephant.gui.titleInfoCurrentLine:SetTextColor(Elephant.defaultConf.colors.currentline.r, Elephant.defaultConf.colors.currentline.g, Elephant.defaultConf.colors.currentline.b, Elephant.defaultConf.colors.currentline.a)
	Elephant.gui.titleInfoCurrentLine:SetPoint("TOPLEFT", Elephant.gui.mainFrame, "TOPLEFT", 175, -15)
	Elephant.gui.titleInfoCurrentLine:SetText("0 / 0")
	
	Elephant.gui.scrollingMessageFrame = CreateFrame("ScrollingMessageFrame", "ElephantScrollingMessageFrame", Elephant.gui.mainFrame)
	Elephant.gui.scrollingMessageFrame:SetFading(false)
	Elephant.gui.scrollingMessageFrame:SetMaxLines(Elephant.defaultConf.scrollmaxlines)
	Elephant.gui.scrollingMessageFrame:SetWidth(525)
	Elephant.gui.scrollingMessageFrame:SetHeight(500)
	Elephant.gui.scrollingMessageFrame:SetPoint("TOPLEFT", 30, -30)
	Elephant.gui.scrollingMessageFrame:EnableMouseWheel(true)
	Elephant.gui.scrollingMessageFrame:CreateFontString()
	Elephant.gui.scrollingMessageFrame:SetFont(Fonts.ChatFontNormal.Font[1], Fonts.ChatFontNormal.Font[2])
	Elephant.gui.scrollingMessageFrame:SetJustifyH("LEFT")
	Elephant.gui.scrollingMessageFrame:SetScript("OnHyperlinkClick", function(_, linkData, link, button) SetItemRef(linkData, link, button) end)
	Elephant.gui.scrollingMessageFrame:SetScript("OnMouseWheel", function(self, delta)
		if delta > 0 then Elephant:Scroll(-Elephant.defaultConf.scrollmouse) end
		if delta < 0 then Elephant:Scroll(Elephant.defaultConf.scrollmouse) end
	end)
	
	local elephantScrollingMessageFrameTexture = CreateFrame("Frame", "ElephantScrollingMessageFrameTexture", Elephant.gui.mainFrame)
	elephantScrollingMessageFrameTexture:SetWidth(535)
	elephantScrollingMessageFrameTexture:SetHeight(510)
	elephantScrollingMessageFrameTexture:SetPoint("TOPLEFT", Elephant.gui.mainFrame, "TOPLEFT", 25, -28)
	elephantScrollingMessageFrameTexture:SetBackdrop ( {
		bgFile = "Interface/ChatFrame/ChatFrameBackground",
		edgeFile = "Interface/Tooltips/UI-Tooltip-Border",
		tile = true,
		tileSize = 16,
		edgeSize = 16,
		insets = { left = 5, right = 5, top = 5, bottom = 5 },
	} )
	elephantScrollingMessageFrameTexture:SetBackdropColor( 0.0, 0.0, 0.0, 0.5 )
	
	local elephantButtonScrollBottom = CreateFrame("Button", "ElephantButtonScrollBottom", Elephant.gui.mainFrame)
	elephantButtonScrollBottom:SetNormalTexture("Interface/PaperDollInfoFrame/UI-Character-SkillsPageDown-Up")
	elephantButtonScrollBottom:SetPushedTexture("Interface/PaperDollInfoFrame/UI-Character-SkillsPageDown-Down")
	elephantButtonScrollBottom:SetHighlightTexture("Interface/Buttons/UI-Common-MouseHilight")
	elephantButtonScrollBottom:SetWidth(32)
	elephantButtonScrollBottom:SetHeight(32)
	elephantButtonScrollBottom:SetPoint("BOTTOMLEFT", Elephant.gui.scrollingMessageFrame, "BOTTOMRIGHT", 4, 0)
	elephantButtonScrollBottom:SetScript("OnEnter", function()
		Elephant:SetTooltip(elephantButtonScrollBottom, L['scroll']['bottom'])
	end)
	elephantButtonScrollBottom:SetScript("OnLeave", function() Elephant:UnsetTooltip() end)
	elephantButtonScrollBottom:SetScript("OnClick", function()
		PlaySound("igChatBottom")
		Elephant.tempConf.currentline = #Elephant.dbpc.char.logs[Elephant.dbpc.char.currentlogindex].logs
		Elephant:ShowCurrentLog()
	end)
	
	local elephantButtonScrollPageDown = CreateFrame("Button", "ElephantButtonScrollPageDown", Elephant.gui.mainFrame)
	elephantButtonScrollPageDown:SetNormalTexture("Interface/ChatFrame/UI-ChatIcon-ScrollDown-Up")
	elephantButtonScrollPageDown:SetPushedTexture("Interface/ChatFrame/UI-ChatIcon-ScrollDown-Down")
	elephantButtonScrollPageDown:SetHighlightTexture("Interface/Buttons/UI-Common-MouseHilight")
	elephantButtonScrollPageDown:SetWidth(32)
	elephantButtonScrollPageDown:SetHeight(32)
	elephantButtonScrollPageDown:SetPoint("BOTTOM", elephantButtonScrollBottom, "TOP", 1, -5)
	elephantButtonScrollPageDown:SetScript("OnEnter", function()
		Elephant:SetTooltip(elephantButtonScrollPageDown, L['scroll']['pagedown'])
	end)
	elephantButtonScrollPageDown:SetScript("OnLeave", function() Elephant:UnsetTooltip() end)
	elephantButtonScrollPageDown:SetScript("OnClick", function()
		PlaySound("igChatScrollDown")
		Elephant:Scroll(Elephant.defaultConf.scrollpage)
	end)
	
	local elephantButtonScrollDown = CreateFrame("Button", "ElephantButtonScrollDown", Elephant.gui.mainFrame)
	elephantButtonScrollDown:SetNormalTexture("Interface/ChatFrame/UI-ChatIcon-ScrollDown-Disabled")
	elephantButtonScrollDown:SetPushedTexture("Interface/ChatFrame/UI-ChatIcon-ScrollDown-Down")
	elephantButtonScrollDown:SetHighlightTexture("Interface/Buttons/UI-Common-MouseHilight")
	elephantButtonScrollDown:SetWidth(32)
	elephantButtonScrollDown:SetHeight(32)
	elephantButtonScrollDown:SetPoint("BOTTOM", elephantButtonScrollPageDown, "TOP", 0, -5)
	elephantButtonScrollDown:SetScript("OnEnter", function()
		Elephant:SetTooltip(elephantButtonScrollDown, L['scroll']['linedown'])
	end)
	elephantButtonScrollDown:SetScript("OnLeave", function() Elephant:UnsetTooltip() end)
	elephantButtonScrollDown:SetScript("OnClick", function()
		PlaySound("igChatScrollDown")
		Elephant:Scroll(1)
	end)
	elephantButtonScrollDown:SetScript("OnUpdate", function()
		if elephantButtonScrollDown:GetButtonState() == "PUSHED" then
			Elephant:Scroll(1)
		end
	end)
	
	local elephantButtonScrollUp = CreateFrame("Button", "ElephantButtonScrollUp", Elephant.gui.mainFrame)
	elephantButtonScrollUp:SetNormalTexture("Interface/ChatFrame/UI-ChatIcon-ScrollUp-Disabled")
	elephantButtonScrollUp:SetPushedTexture("Interface/ChatFrame/UI-ChatIcon-ScrollUp-Down")
	elephantButtonScrollUp:SetHighlightTexture("Interface/Buttons/UI-Common-MouseHilight")
	elephantButtonScrollUp:SetWidth(32)
	elephantButtonScrollUp:SetHeight(32)
	elephantButtonScrollUp:SetPoint("BOTTOM", elephantButtonScrollDown, "TOP", 0, -5)
	elephantButtonScrollUp:SetScript("OnEnter", function()
		Elephant:SetTooltip(elephantButtonScrollUp, L['scroll']['lineup'])
	end)
	elephantButtonScrollUp:SetScript("OnLeave", function() Elephant:UnsetTooltip() end)
	elephantButtonScrollUp:SetScript("OnClick", function()
		PlaySound("igChatScrollUp")
		Elephant:Scroll(-1)
	end)
	elephantButtonScrollUp:SetScript("OnUpdate", function()
		if elephantButtonScrollUp:GetButtonState() == "PUSHED" then
			Elephant:Scroll(-1)
		end
	end)
	
	local elephantButtonScrollPageUp = CreateFrame("Button", "ElephantButtonScrollPageUp", Elephant.gui.mainFrame)
	elephantButtonScrollPageUp:SetNormalTexture("Interface/ChatFrame/UI-ChatIcon-ScrollUp-Up")
	elephantButtonScrollPageUp:SetPushedTexture("Interface/ChatFrame/UI-ChatIcon-ScrollUp-Down")
	elephantButtonScrollPageUp:SetHighlightTexture("Interface/Buttons/UI-Common-MouseHilight")
	elephantButtonScrollPageUp:SetWidth(32)
	elephantButtonScrollPageUp:SetHeight(32)
	elephantButtonScrollPageUp:SetPoint("BOTTOM", elephantButtonScrollUp, "TOP", 0, -5)
	elephantButtonScrollPageUp:SetScript("OnEnter",
		function() Elephant:SetTooltip(elephantButtonScrollPageUp, L['scroll']['pageup'])
	end)
	elephantButtonScrollPageUp:SetScript("OnLeave", function() Elephant:UnsetTooltip() end)
	elephantButtonScrollPageUp:SetScript("OnClick", function()
		PlaySound("igChatScrollUp")
		Elephant:Scroll(-Elephant.defaultConf.scrollpage)
	end)
	
	local elephantButtonScrollTop = CreateFrame("Button", "ElephantButtonScrollTop", Elephant.gui.mainFrame)
	elephantButtonScrollTop:SetNormalTexture("Interface/PaperDollInfoFrame/UI-Character-SkillsPageUp-Up")
	elephantButtonScrollTop:SetPushedTexture("Interface/PaperDollInfoFrame/UI-Character-SkillsPageUp-Down")
	elephantButtonScrollTop:SetHighlightTexture("Interface/Buttons/UI-Common-MouseHilight")
	elephantButtonScrollTop:SetWidth(32)
	elephantButtonScrollTop:SetHeight(32)
	elephantButtonScrollTop:SetPoint("BOTTOM", elephantButtonScrollPageUp, "TOP", -1, -5)
	elephantButtonScrollTop:SetScript("OnEnter", function()
		Elephant:SetTooltip(elephantButtonScrollTop, L['scroll']['top'])
	end)
	elephantButtonScrollTop:SetScript("OnLeave", function() Elephant:UnsetTooltip() end)
	elephantButtonScrollTop:SetScript("OnClick", function()
		PlaySound("igChatBottom")
		Elephant.tempConf.currentline = #Elephant.dbpc.char.logs[Elephant.dbpc.char.currentlogindex].logs
		if Elephant.tempConf.currentline > 1 then
			Elephant.tempConf.currentline = 1
		end
		Elephant:ShowCurrentLog()
	end)
	
	Elephant.gui.elephantButtonCatchOptions = CreateFrame("Button", "ElephantButtonCatchOptions", Elephant.gui.mainFrame)
	Elephant.gui.elephantButtonCatchOptions:SetNormalTexture("Interface/ChatFrame/UI-ChatIcon-Chat-Up")
	Elephant.gui.elephantButtonCatchOptions:SetPushedTexture("Interface/ChatFrame/UI-ChatIcon-Chat-Down")
	Elephant.gui.elephantButtonCatchOptions:SetHighlightTexture("Interface/Buttons/UI-Common-MouseHilight")
	Elephant.gui.elephantButtonCatchOptions:SetDisabledTexture("Interface/ChatFrame/UI-ChatIcon-Chat-Disabled")
	Elephant.gui.elephantButtonCatchOptions:SetWidth(32)
	Elephant.gui.elephantButtonCatchOptions:SetHeight(32)
	Elephant.gui.elephantButtonCatchOptions:SetPoint("TOPLEFT", Elephant.gui.scrollingMessageFrame, "TOPRIGHT", 5, 0)
	Elephant.gui.elephantButtonCatchOptions:SetScript("OnEnter", function()
		Elephant:SetTooltip(Elephant.gui.elephantButtonCatchOptions, L['catchers'])
	end)
	Elephant.gui.elephantButtonCatchOptions:SetScript("OnLeave", function() Elephant:UnsetTooltip() end)
	Elephant.gui.elephantButtonCatchOptions:SetScript("OnClick", function()
		ToggleDropDownMenu(1, nil, Elephant.dropdowns.catchOptions, Elephant.gui.elephantButtonCatchOptions, 0, 0)
	end)
	
	local elephantButtonTabs = {}
	elephantButtonTabs.elephantButtonWhisperTab = CreateFrame("Button", "ElephantButtonWhisperTab", Elephant.gui.mainFrame)
	elephantButtonTabs.elephantButtonWhisperTab:SetNormalTexture("Interface/Minimap/Tracking/None")
	--elephantButtonTabs.elephantButtonWhisperTab:SetPushedTexture("Interface/Icons/Ability_Stealth")
	elephantButtonTabs.elephantButtonWhisperTab:SetHighlightTexture("Interface/Minimap/Tracking/None")
	elephantButtonTabs.elephantButtonWhisperTab:SetWidth(23)
	elephantButtonTabs.elephantButtonWhisperTab:SetHeight(23)
	elephantButtonTabs.elephantButtonWhisperTab:SetPoint("TOP", Elephant.gui.elephantButtonCatchOptions, "BOTTOM", 0, -5)
	elephantButtonTabs.elephantButtonWhisperTab:SetScript("OnEnter", function()
		Elephant:SetTooltip(elephantButtonTabs.elephantButtonWhisperTab, {L['chatnames']['whisper'], Elephant:GetEnabledMsg(Elephant.dbpc.char.logs[Elephant.defaultConf.defaultindexes.whisper].enabled), format(L['nblines'], #Elephant.dbpc.char.logs[Elephant.defaultConf.defaultindexes.whisper].logs)})
	end)
	elephantButtonTabs.elephantButtonWhisperTab:SetScript("OnLeave", function() Elephant:UnsetTooltip() end)
	elephantButtonTabs.elephantButtonWhisperTab:SetScript("OnClick", function() Elephant:ChangeLog(Elephant.defaultConf.defaultindexes.whisper) end)
	
	elephantButtonTabs.elephantButtonRaidTab = CreateFrame("Button", "ElephantButtonRaidTab", Elephant.gui.mainFrame)
	elephantButtonTabs.elephantButtonRaidTab:SetNormalTexture("Interface/Minimap/Tracking/Ammunition")
	--elephantButtonTabs.elephantButtonRaidTab:SetPushedTexture("Interface/Icons/INV_Misc_Head_Dragon_01")
	elephantButtonTabs.elephantButtonRaidTab:SetHighlightTexture("Interface/Minimap/Tracking/Ammunition")
	elephantButtonTabs.elephantButtonRaidTab:SetWidth(23)
	elephantButtonTabs.elephantButtonRaidTab:SetHeight(23)
	elephantButtonTabs.elephantButtonRaidTab:SetPoint("TOP", elephantButtonTabs.elephantButtonWhisperTab, "BOTTOM")
	elephantButtonTabs.elephantButtonRaidTab:SetScript("OnEnter", function()
		Elephant:SetTooltip(elephantButtonTabs.elephantButtonRaidTab, {L['chatnames']['raid'], Elephant:GetEnabledMsg(Elephant.dbpc.char.logs[Elephant.defaultConf.defaultindexes.raid].enabled), format(L['nblines'], #Elephant.dbpc.char.logs[Elephant.defaultConf.defaultindexes.raid].logs)})
	end)
	elephantButtonTabs.elephantButtonRaidTab:SetScript("OnLeave", function() Elephant:UnsetTooltip() end)
	elephantButtonTabs.elephantButtonRaidTab:SetScript("OnClick", function() Elephant:ChangeLog(Elephant.defaultConf.defaultindexes.raid) end)
	
	elephantButtonTabs.elephantButtonPartyTab = CreateFrame("Button", "ElephantButtonPartyTab", Elephant.gui.mainFrame)
	elephantButtonTabs.elephantButtonPartyTab:SetNormalTexture("Interface/GroupFrame/UI-Group-LeaderIcon")
	--elephantButtonTabs.elephantButtonPartyTab:SetPushedTexture("Interface/Icons/Spell_Holy_PrayerofSpirit")
	elephantButtonTabs.elephantButtonPartyTab:SetHighlightTexture("Interface/GroupFrame/UI-Group-LeaderIcon")
	elephantButtonTabs.elephantButtonPartyTab:SetWidth(23)
	elephantButtonTabs.elephantButtonPartyTab:SetHeight(23)
	elephantButtonTabs.elephantButtonPartyTab:SetPoint("TOP", elephantButtonTabs.elephantButtonRaidTab, "BOTTOM")
	elephantButtonTabs.elephantButtonPartyTab:SetScript("OnEnter", function()
		Elephant:SetTooltip(elephantButtonTabs.elephantButtonPartyTab, {L['chatnames']['party'], Elephant:GetEnabledMsg(Elephant.dbpc.char.logs[Elephant.defaultConf.defaultindexes.party].enabled), format(L['nblines'], #Elephant.dbpc.char.logs[Elephant.defaultConf.defaultindexes.party].logs)})
	end)
	elephantButtonTabs.elephantButtonPartyTab:SetScript("OnLeave", function() Elephant:UnsetTooltip() end)
	elephantButtonTabs.elephantButtonPartyTab:SetScript("OnClick", function() Elephant:ChangeLog(Elephant.defaultConf.defaultindexes.party) end)
	
	elephantButtonTabs.elephantButtonSayTab = CreateFrame("Button", "ElephantButtonSayTab", Elephant.gui.mainFrame)
	elephantButtonTabs.elephantButtonSayTab:SetNormalTexture("Interface/GossipFrame/GossipGossipIcon")
	--elephantButtonTabs.elephantButtonSayTab:SetPushedTexture("Interface/Icons/INV_Misc_Bomb_04")
	elephantButtonTabs.elephantButtonSayTab:SetHighlightTexture("Interface/GossipFrame/GossipGossipIcon")
	elephantButtonTabs.elephantButtonSayTab:SetWidth(23)
	elephantButtonTabs.elephantButtonSayTab:SetHeight(23)
	elephantButtonTabs.elephantButtonSayTab:SetPoint("TOP", elephantButtonTabs.elephantButtonPartyTab, "BOTTOM")
	elephantButtonTabs.elephantButtonSayTab:SetScript("OnEnter", function()
		Elephant:SetTooltip(elephantButtonTabs.elephantButtonSayTab, {L.chatnames.say, Elephant:GetEnabledMsg(Elephant.dbpc.char.logs[Elephant.defaultConf.defaultindexes.say].enabled), format(L['nblines'], #Elephant.dbpc.char.logs[Elephant.defaultConf.defaultindexes.say].logs)})
	end)
	elephantButtonTabs.elephantButtonSayTab:SetScript("OnLeave", function() Elephant:UnsetTooltip() end)
	elephantButtonTabs.elephantButtonSayTab:SetScript("OnClick", function() Elephant:ChangeLog(Elephant.defaultConf.defaultindexes.say) end)
	
	elephantButtonTabs.elephantButtonYellTab = CreateFrame("Button", "ElephantButtonYellTab", Elephant.gui.mainFrame)
	elephantButtonTabs.elephantButtonYellTab:SetNormalTexture("Interface/Buttons/UI-GuildButton-MOTD-Up")
	--elephantButtonTabs.elephantButtonYellTab:SetPushedTexture("Interface/Icons/Spell_Fire_Incinerate")
	elephantButtonTabs.elephantButtonYellTab:SetHighlightTexture("Interface/Buttons/UI-GuildButton-MOTD-Up")
	elephantButtonTabs.elephantButtonYellTab:SetWidth(23)
	elephantButtonTabs.elephantButtonYellTab:SetHeight(23)
	elephantButtonTabs.elephantButtonYellTab:SetPoint("TOP", elephantButtonTabs.elephantButtonSayTab, "BOTTOM")
	elephantButtonTabs.elephantButtonYellTab:SetScript("OnEnter", function()
		Elephant:SetTooltip(elephantButtonTabs.elephantButtonYellTab, {L['chatnames']['yell'], Elephant:GetEnabledMsg(Elephant.dbpc.char.logs[Elephant.defaultConf.defaultindexes.yell].enabled), format(L['nblines'], #Elephant.dbpc.char.logs[Elephant.defaultConf.defaultindexes.yell].logs)})
	end)
	elephantButtonTabs.elephantButtonYellTab:SetScript("OnLeave", function() Elephant:UnsetTooltip() end)
	elephantButtonTabs.elephantButtonYellTab:SetScript("OnClick", function() Elephant:ChangeLog(Elephant.defaultConf.defaultindexes.yell) end)
	
	elephantButtonTabs.elephantButtonOfficerTab = CreateFrame("Button", "ElephantButtonOfficerTab", Elephant.gui.mainFrame)
	elephantButtonTabs.elephantButtonOfficerTab:SetNormalTexture("Interface/GossipFrame/AvailableQuestIcon")
	--elephantButtonTabs.elephantButtonOfficerTab:SetPushedTexture("Interface/Icons/Spell_Nature_EnchantArmor")
	elephantButtonTabs.elephantButtonOfficerTab:SetHighlightTexture("Interface/GossipFrame/AvailableQuestIcon")
	elephantButtonTabs.elephantButtonOfficerTab:SetWidth(23)
	elephantButtonTabs.elephantButtonOfficerTab:SetHeight(23)
	elephantButtonTabs.elephantButtonOfficerTab:SetPoint("TOP", elephantButtonTabs.elephantButtonYellTab, "BOTTOM")
	elephantButtonTabs.elephantButtonOfficerTab:SetScript("OnEnter", function()
		Elephant:SetTooltip(elephantButtonTabs.elephantButtonOfficerTab, {L['chatnames']['officer'], Elephant:GetEnabledMsg(Elephant.dbpc.char.logs[Elephant.defaultConf.defaultindexes.officer].enabled), format(L['nblines'], #Elephant.dbpc.char.logs[Elephant.defaultConf.defaultindexes.officer].logs)})
	end)
	elephantButtonTabs.elephantButtonOfficerTab:SetScript("OnLeave", function() Elephant:UnsetTooltip() end)
	elephantButtonTabs.elephantButtonOfficerTab:SetScript("OnClick", function() Elephant:ChangeLog(Elephant.defaultConf.defaultindexes.officer) end)
	
	elephantButtonTabs.elephantButtonGuildTab = CreateFrame("Button", "ElephantButtonGuildTab", Elephant.gui.mainFrame)
	elephantButtonTabs.elephantButtonGuildTab:SetNormalTexture("Interface/Minimap/Tracking/Food")
	--elephantButtonTabs.elephantButtonGuildTab:SetPushedTexture("Interface/Icons/INV_Shirt_GuildTabard_01")
	elephantButtonTabs.elephantButtonGuildTab:SetHighlightTexture("Interface/Minimap/Tracking/Food")
	elephantButtonTabs.elephantButtonGuildTab:SetWidth(23)
	elephantButtonTabs.elephantButtonGuildTab:SetHeight(23)
	elephantButtonTabs.elephantButtonGuildTab:SetPoint("TOP", elephantButtonTabs.elephantButtonOfficerTab, "BOTTOM")
	elephantButtonTabs.elephantButtonGuildTab:SetScript("OnEnter", function()
		Elephant:SetTooltip(elephantButtonTabs.elephantButtonGuildTab, {L['chatnames']['guild'], Elephant:GetEnabledMsg(Elephant.dbpc.char.logs[Elephant.defaultConf.defaultindexes.guild].enabled), format(L['nblines'], #Elephant.dbpc.char.logs[Elephant.defaultConf.defaultindexes.guild].logs)})
	end)
	elephantButtonTabs.elephantButtonGuildTab:SetScript("OnLeave", function() Elephant:UnsetTooltip() end)
	elephantButtonTabs.elephantButtonGuildTab:SetScript("OnClick", function() Elephant:ChangeLog(Elephant.defaultConf.defaultindexes.guild) end)
	
	--[[ Custom chats & dropdown ]]
	elephantButtonTabs.elephantButtonCustomTab = CreateFrame("Button", "ElephantButtonCustomTab", Elephant.gui.mainFrame)
	elephantButtonTabs.elephantButtonCustomTab:SetNormalTexture("Interface/Cursor/Interact")
	--elephantButtonTabs.elephantButtonCustomTab:SetPushedTexture("Interface/Icons/Trade_Engineering")
	elephantButtonTabs.elephantButtonCustomTab:SetHighlightTexture("Interface/Cursor/Interact")
	elephantButtonTabs.elephantButtonCustomTab:SetWidth(23)
	elephantButtonTabs.elephantButtonCustomTab:SetHeight(23)
	elephantButtonTabs.elephantButtonCustomTab:SetPoint("TOP", elephantButtonTabs.elephantButtonGuildTab, "BOTTOM", 0, -25)
	elephantButtonTabs.elephantButtonCustomTab:SetScript("OnEnter", function()
		Elephant:SetTooltip(elephantButtonTabs.elephantButtonCustomTab, {L['chatnames']['custom']})
	end)
	elephantButtonTabs.elephantButtonCustomTab:SetScript("OnLeave", function() Elephant:UnsetTooltip() end)
	elephantButtonTabs.elephantButtonCustomTab:SetScript("OnClick", function()
		ToggleDropDownMenu(1, nil, Elephant.dropdowns.customChats, elephantButtonTabs.elephantButtonCustomTab, 0, 0)
	end)
	
	--[[ General chats & dropdown ]]
	elephantButtonTabs.elephantButtonGeneralTab = CreateFrame("Button", "ElephantButtonGeneralTab", Elephant.gui.mainFrame)
	elephantButtonTabs.elephantButtonGeneralTab:SetNormalTexture("Interface/Cursor/Speak")
	--elephantButtonTabs.elephantButtonGeneralTab:SetPushedTexture("Interface/Icons/Spell_Shadow_SoulGem")
	elephantButtonTabs.elephantButtonGeneralTab:SetHighlightTexture("Interface/Cursor/Speak")
	elephantButtonTabs.elephantButtonGeneralTab:SetWidth(23)
	elephantButtonTabs.elephantButtonGeneralTab:SetHeight(23)
	elephantButtonTabs.elephantButtonGeneralTab:SetPoint("TOP", elephantButtonTabs.elephantButtonCustomTab, "BOTTOM")
	elephantButtonTabs.elephantButtonGeneralTab:SetScript("OnEnter", function()
		Elephant:SetTooltip(elephantButtonTabs.elephantButtonGeneralTab, {L['chatnames']['general']})
	end)
	elephantButtonTabs.elephantButtonGeneralTab:SetScript("OnLeave", function() Elephant:UnsetTooltip() end)
	elephantButtonTabs.elephantButtonGeneralTab:SetScript("OnClick", function()
		ToggleDropDownMenu(1, nil, Elephant.dropdowns.generalChats, elephantButtonTabs.elephantButtonGeneralTab, 0, 0)
	end)
	
	--[[ Misc. dropdown ]]
	elephantButtonTabs.elephantButtonMiscTab = CreateFrame("Button", "ElephantButtonMiscTab", Elephant.gui.mainFrame)
	elephantButtonTabs.elephantButtonMiscTab:SetNormalTexture("Interface/Cursor/Innkeeper")
	--elephantButtonTabs.elephantButtonMiscTab:SetPushedTexture("Interface/Icons/Spell_Frost_WindWalkOn")
	elephantButtonTabs.elephantButtonMiscTab:SetHighlightTexture("Interface/Cursor/Innkeeper")
	elephantButtonTabs.elephantButtonMiscTab:SetWidth(23)
	elephantButtonTabs.elephantButtonMiscTab:SetHeight(23)
	elephantButtonTabs.elephantButtonMiscTab:SetPoint("TOP", elephantButtonTabs.elephantButtonGeneralTab, "BOTTOM")
	elephantButtonTabs.elephantButtonMiscTab:SetScript("OnEnter", function()
		Elephant:SetTooltip(elephantButtonTabs.elephantButtonMiscTab, {L['chatnames']['misc']})
	end)
	elephantButtonTabs.elephantButtonMiscTab:SetScript("OnLeave", function() Elephant:UnsetTooltip() end)
	elephantButtonTabs.elephantButtonMiscTab:SetScript("OnClick", function()
		ToggleDropDownMenu(1, nil, Elephant.dropdowns.miscChats, elephantButtonTabs.elephantButtonMiscTab, 0, 0)
	end)
	
	Elephant.gui.buttonDelete = CreateFrame("Button", "ElephantButtonDelete", Elephant.gui.mainFrame, OptionsButtonTemplate)
	Elephant.gui.buttonDelete:SetNormalTexture("Interface/Buttons/UI-DialogBox-Button-Up")
	Elephant.gui.buttonDelete:SetPushedTexture("Interface/Buttons/UI-DialogBox-Button-Down")
	Elephant.gui.buttonDelete:SetHighlightTexture("Interface/Buttons/UI-DialogBox-Button-Highlight")
	Elephant.gui.buttonDelete:SetDisabledTexture("Interface/Buttons/UI-DialogBox-Button-Disabled")
	Elephant.gui.buttonDelete:SetWidth(75)
	Elephant.gui.buttonDelete:SetHeight(33)
	Elephant.gui.buttonDelete:SetPoint("BOTTOMLEFT", Elephant.gui.mainFrame, 20, 5)
	Elephant.gui.buttonDelete:SetFontString(Elephant.gui.buttonDelete:CreateFontString())
	Elephant.gui.buttonDelete:GetFontString():SetPoint("TOP", Elephant.gui.buttonDelete, "TOP", 0, -5)
	Elephant.gui.buttonDelete:GetFontString():SetFont(Fonts.GameFontNormal.Font[1], Fonts.GameFontNormal.Font[2], "OUTLINE")
	Elephant.gui.buttonDelete:GetFontString():SetText(DELETE)
	Elephant.gui.buttonDelete:SetScript("OnClick", function()
		Elephant:DeleteCurrentLog()
		Elephant:ChangeLog(Elephant.defaultConf.defaultlogindex)
		Elephant:ShowCurrentLog()
	end)
	
	Elephant.gui.buttonEnable = CreateFrame("Button", "ElephantButtonEnable", Elephant.gui.mainFrame, OptionsButtonTemplate)
	Elephant.gui.buttonEnable:SetNormalTexture("Interface/Buttons/UI-DialogBox-Button-Up")
	Elephant.gui.buttonEnable:SetPushedTexture("Interface/Buttons/UI-DialogBox-Button-Down")
	Elephant.gui.buttonEnable:SetHighlightTexture("Interface/Buttons/UI-DialogBox-Button-Highlight")
	Elephant.gui.buttonEnable:SetWidth(75)
	Elephant.gui.buttonEnable:SetHeight(33)
	Elephant.gui.buttonEnable:SetPoint("BOTTOM", Elephant.gui.mainFrame, 0, 5)
	Elephant.gui.buttonEnable:SetFontString(Elephant.gui.buttonEnable:CreateFontString())
	Elephant.gui.buttonEnable:GetFontString():SetPoint("TOP", Elephant.gui.buttonEnable, "TOP", 0, -5)
	Elephant.gui.buttonEnable:GetFontString():SetFont(Fonts.GameFontNormal.Font[1], Fonts.GameFontNormal.Font[2], "OUTLINE")
	Elephant.gui.buttonEnable:GetFontString():SetText(ENABLE)
	Elephant.gui.buttonEnable:SetScript("OnClick", function() Elephant:ToggleEnableCurrentLog() end)
	
	local elephantButtonEmpty = CreateFrame("Button", "ElephantButtonEmpty", Elephant.gui.mainFrame, OptionsButtonTemplate)
	elephantButtonEmpty:SetNormalTexture("Interface/Buttons/UI-DialogBox-Button-Up")
	elephantButtonEmpty:SetPushedTexture("Interface/Buttons/UI-DialogBox-Button-Down")
	elephantButtonEmpty:SetHighlightTexture("Interface/Buttons/UI-DialogBox-Button-Highlight")
	elephantButtonEmpty:SetWidth(75)
	elephantButtonEmpty:SetHeight(33)
	elephantButtonEmpty:SetPoint("RIGHT", Elephant.gui.buttonEnable, "LEFT")
	elephantButtonEmpty:SetFontString(elephantButtonEmpty:CreateFontString())
	elephantButtonEmpty:GetFontString():SetPoint("TOP", elephantButtonEmpty, "TOP", 0, -5)
	elephantButtonEmpty:GetFontString():SetFont(Fonts.GameFontNormal.Font[1], Fonts.GameFontNormal.Font[2], "OUTLINE")
	elephantButtonEmpty:GetFontString():SetText(L['Empty'])
	elephantButtonEmpty:SetScript("OnClick", function() StaticPopup_Show("ELEPHANT_EMPTY") end)
	
	Elephant.gui.buttonCopy = CreateFrame("Button", "ElephantButtonCopy", Elephant.gui.mainFrame, OptionsButtonTemplate)
	Elephant.gui.buttonCopy:SetNormalTexture("Interface/Buttons/UI-DialogBox-Button-Up")
	Elephant.gui.buttonCopy:SetPushedTexture("Interface/Buttons/UI-DialogBox-Button-Down")
	Elephant.gui.buttonCopy:SetHighlightTexture("Interface/Buttons/UI-DialogBox-Button-Highlight")
	Elephant.gui.buttonCopy:SetDisabledTexture("Interface/Buttons/UI-DialogBox-Button-Disabled")
	Elephant.gui.buttonCopy:SetWidth(75)
	Elephant.gui.buttonCopy:SetHeight(33)
	Elephant.gui.buttonCopy:SetPoint("LEFT", Elephant.gui.buttonEnable, "RIGHT")
	Elephant.gui.buttonCopy:SetFontString(Elephant.gui.buttonCopy:CreateFontString())
	Elephant.gui.buttonCopy:GetFontString():SetPoint("TOP", Elephant.gui.buttonCopy, "TOP", 0, -5)
	Elephant.gui.buttonCopy:GetFontString():SetFont(Fonts.GameFontNormal.Font[1], Fonts.GameFontNormal.Font[2], "OUTLINE")
	Elephant.gui.buttonCopy:GetFontString():SetText(L['copy'])
	Elephant.gui.buttonCopy:SetScript("OnClick", function() Elephant:ShowCopyWindow() end)
	
	local elephantButtonClose = CreateFrame("Button", "ElephantButtonClose", Elephant.gui.mainFrame, OptionsButtonTemplate)
	elephantButtonClose:SetNormalTexture("Interface/Buttons/UI-DialogBox-Button-Up")
	elephantButtonClose:SetPushedTexture("Interface/Buttons/UI-DialogBox-Button-Down")
	elephantButtonClose:SetHighlightTexture("Interface/Buttons/UI-DialogBox-Button-Highlight")
	elephantButtonClose:SetWidth(75)
	elephantButtonClose:SetHeight(33)
	elephantButtonClose:SetPoint("BOTTOMRIGHT", -20, 5)
	elephantButtonClose:SetFontString(elephantButtonClose:CreateFontString())
	elephantButtonClose:GetFontString():SetPoint("TOP", elephantButtonClose, "TOP", 0, -5)
	elephantButtonClose:GetFontString():SetFont(Fonts.GameFontNormal.Font[1], Fonts.GameFontNormal.Font[2], "OUTLINE")
	elephantButtonClose:GetFontString():SetText(CLOSE)
	elephantButtonClose:SetScript("OnClick", function() Elephant:Toggle() end)
	
	Elephant:SetTitleInfoMaxLog()
	
	Elephant.gui.mainFrame:Hide()
	
	--[[ Copy window ]]
	-- TODO: Check if this is still required
	-- TODO: Check values as well
	Elephant.gui.copyFrameScrollFrameEditBox = getglobal("ElephantCopyFrameScrollFrameEditBox")
	Elephant.gui.elephantCopyFrameButtonBBCode = getglobal("ElephantCopyFrameBBCodeButton")
	
	--[[ Elephant button ]]
	if Elephant.db.profile.button == true then
		Elephant:CreateButton()
	end
	
	--[[ Creating own slashcommands ]]
	local options = {
		type = 'group',
		args = {
			togglebutton = {
				type = 'toggle',
				name = L['showbutton'],
				desc = L['showbutton_desc'],
				get = function()
					return Elephant.db.profile.button
				end,
				set = function()
					Elephant:ToggleButton()
				end,
			},
			-- Deactivating prat options (see RegisterEventsRefresh() in Event.lua)
			--[[
			prat = {
				type = 'toggle',
				name = L['prat'],
				desc = L['prat_desc'],
				get = function()
					return Elephant.db.profile.prat
				end,
				set = function(_, v)
					Elephant.db.profile.prat = v
					Elephant:RegisterEventsRefresh()
				end,
				disabled = function()
					return not Prat
				end,
			},
			forcetimestamp = {
				type = 'toggle',
				name = L['forcetimestamp'],
				desc = L['forcetimestamp_desc'],
				get = function()
					return Elephant.db.profile.forcetimestamp
				end,
				set = function(_, v)
					Elephant.db.profile.forcetimestamp = v
				end,
				disabled = function()
					return (not Prat) or (not Elephant.db.profile.prat)
				end,
			},
			]]
			filters = {
				type = 'group',
				name = L['Filters'],
				desc = L['Filters_desc'],
				args = {
					add = {
						type = 'input',
						name = NEW,
						desc = L['newfilter_desc'],
						get = false,
						set = function(_, input)
							if string.match(input, L['filterregex']) == nil then
								Elephant:Print(format(L['filtererror'], input))
							else
								Elephant:AddFilter(input)
							end
						end,
						usage = L['filterusage'],
					},
					delete = {
						type = 'select',
						name = DELETE,
						desc = L['deletefilter_desc'],
						set = function(_,filterindex)
							Elephant:DeleteFilter(filterindex)
						end,
						values = function()
							return 	Elephant.db.profile.filters
						end,
						hidden = function()
							return (not Elephant.db.profile.filters) or
								(#Elephant.db.profile.filters == 0)
						end
					}
				},
			},
			log = {
				type = 'group',
				name = L['logs'],
				desc = L['logs_desc'],
				args = {
					clearall = {
						type = 'execute',
						name = L['clearallhelp'],
						desc = L['clearallhelp_desc'],
						func = function()
							StaticPopup_Show("ELEPHANT_CLEARALL")
						end,
					},
					default = {
						type = 'toggle',
						name = L['enableddefault'],
						desc = L['enableddefault_desc'],
						get = function()
							return Elephant.db.profile.defaultlog
						end,
						set = function(_, isEnabled)
							Elephant.db.profile.defaultlog = isEnabled
						end,
					},
					max = {
						type = 'range',
						name = L['maxlogwords'],
						desc = L['maxlogwords_desc'],
						min = Elephant.defaultConf.minlogsize,
						max = Elephant.defaultConf.maxlogsize,
						step = 1,
						get = function()
							return Elephant.db.profile.maxlog
						end,
						set = function(_, nb)
							Elephant:ChangeMaxLog(nb)
						end,
					},
					files = {
						type = 'group',
						name = L['files'],
						desc = L['files_desc'],
						args = {
							activate = {
								type = 'toggle',
								name = L['activate'],
								desc = L['activate_desc'],
								get = function()
									return Elephant.db.profile.activate_log
								end,
								set = function(_, nv)
									Elephant.db.profile.activate_log = nv
									Elephant:ChatLogEnable(Elephant.db.profile.chatlog)
									Elephant:CombatLogEnable(Elephant.db.profile.combatlog)
								end,
							},
							chat = {
								type = 'toggle',
								name = L['chatlog'],
								desc = L['chatlog_desc'],
								get = function()
									return Elephant.db.profile.chatlog
								end,
								set = function(_, isEnabled)
									Elephant.db.profile.chatlog = isEnabled
									Elephant:ChatLogEnable(isEnabled)
								end,
								hidden = function()
									return not Elephant.db.profile.activate_log
								end,
							},
							combat = {
								type = 'toggle',
								name = L['combatlog'],
								desc = L['combatlog_desc'],
								get = function()
									return Elephant.db.profile.combatlog
								end,
								set = function(_, isEnabled)
									Elephant.db.profile.combatlog = isEnabled
									Elephant:CombatLogEnable(isEnabled)
								end,
								hidden = function()
									return not Elephant.db.profile.activate_log
								end,
							},
						},
					},
				},
			},
			reset = {
				type = 'group',
				name = L['reset'],
				desc = L['reset_desc'],
				args = {
					all = {
						type = 'execute',
						name = L['resethelp'],
						desc = L['resethelp_desc'],
						func = function()
							StaticPopup_Show("ELEPHANT_RESET")
						end,
					},
					position = {
						type = 'execute',
						name = L['resetloc'],
						desc = L['resetloc_desc'],
						func = function()
							Elephant:ResetPosition()
						end,
					},
				},
			},
		}
	}
	
	LibStub("AceConfig-3.0"):RegisterOptionsTable("Elephant", options)
	LibStub("AceConfigDialog-3.0"):AddToBlizOptions("Elephant")
	Elephant:RegisterChatCommand("elephant", function()
		InterfaceOptionsFrame_OpenToCategory("Elephant")
	end)
	
	--[[ Miscelleaneous elements status at loading ]]
	Elephant:ChatLogEnable(Elephant.db.profile.chatlog)
	Elephant:CombatLogEnable(Elephant.db.profile.combatlog)
	
	--[[ Checks & creates default log structures ]]
	Elephant:InitDefaultLogStructures()
	Elephant:AddHeaderToStructures()
end

function Elephant:OnEnable()
	--[[ Registering events ]]
	Elephant:RegisterEventsRefresh()
	
	--[[ Displays default log ]]
	if not Elephant.dbpc.char.logs[Elephant.dbpc.char.currentlogindex] then
		Elephant.dbpc.char.currentlogindex = Elephant.defaultConf.defaultlogindex
	end
	Elephant.tempConf.currentline = #Elephant.dbpc.char.logs[Elephant.dbpc.char.currentlogindex].logs
	Elephant:ShowCurrentLog()
end