local L = LibStub("AceLocale-3.0"):GetLocale("Elephant")
local Elephant = LibStub("AceAddon-3.0"):GetAddon("Elephant")

function Elephant:InitDefaultLogStructures()
	local k,v
	for k,v in pairs(Elephant.defaultConf.defaultindexes) do
		if not Elephant.dbpc.char.logs[v] then
			Elephant:CreateNewLogStructure(v, Elephant.defaultConf.defaultnames[k])
		end
	end
	
	for k,v in pairs(L['generalchats']) do
		if not Elephant.dbpc.char.logs[k] then
			Elephant:CreateNewLogStructure(k, v.name)
		end
	end
end

function Elephant:InitCustomStructures()
	local channelCustomId,channelName
	local found
	local i
	-- Max: 20 channels
	for i=1,20 do
		_,channelName = GetChannelName(i)
		
		if channelName ~= nil then
			channelCustomId = string.lower(channelName)
			
			found = false
			for k,v in pairs(L['generalchats']) do
				if (channelCustomId == k) or string.find(channelCustomId, k .. " - ") then
					found = true
					break
				end
			end
			
			if not found then
				Elephant:InitCustomStructure(string.lower(channelName), channelName)
			end
		end
	end
end

function Elephant:InitCustomStructure(id, name)
	if Elephant.dbpc.char.logs[id] then
		return
	end
	
	Elephant:CreateNewLogStructure(id, name)
	if Elephant.dbpc.char.logs[id].enabled then
		Elephant:AddHeaderToTable(Elephant.dbpc.char.logs[id])
	end
end

function Elephant:CreateNewLogStructure(index, name)
	Elephant.dbpc.char.logs[index] = {}
	
	Elephant.dbpc.char.logs[index].name = name
	Elephant.dbpc.char.logs[index].enabled = Elephant.db.profile.defaultlog
	Elephant.dbpc.char.logs[index].logs = {}
end

function Elephant:AddHeaderToStructures(onlyGeneralChannels)
	local k, v
	
	for k,v in pairs(Elephant.dbpc.char.logs) do
		if v.enabled then
			if
				onlyGeneralChannels == nil or
				onlyGeneralChannels == false or (
					onlyGeneralChannels == true and (
						type(k) == "number" or
						L['generalchats'][k]
					)
				)
			then
				Elephant:AddHeaderToTable(v)
			end
		end
		Elephant:CheckTableSize(k)
	end
end

--[[
	Should only be used if the log of this chat IS enabled
	
	@param logObject a log object
]]--
function Elephant:AddHeaderToTable(logObject)
	-- For "easier-to-read" code
	local t = logObject.logs
	
	-- If log is not empty
	if #t > 0 then
		if not logObject.hasMessage then
			-- If the log does not have a message since the last header
			-- then simply modify the last header
			Elephant:SaveMsgToTableAtPosition(t, { ['type'] = "SYSTEM", ['arg1'] = Elephant:GetEnableActionMsg(true) } , #t)
		else
			-- Otherwise add two lines
			Elephant:AddMsgToTable(t, { ['arg1'] = " " } )
			Elephant:AddMsgToTable(t, { ['arg1'] = " " } )
		end
	end

	-- If log did  save a message since the last header
	-- or log is empty
	if logObject.hasMessage or #t == 0 then
		Elephant:AddMsgToTable(t, { ['type'] = "SYSTEM", ['arg1'] = Elephant:GetEnableActionMsg(true) } )
	end
	
	-- Specify that no messages has been saved since the last header
	if logObject.hasMessage then
		logObject.hasMessage = false
	end
end

function Elephant:AddMsgToTable(t, msg)
	t[#t+1] = msg
end

function Elephant:SaveMsgToTableAtPosition(t, msg, position)
	t[position] = msg
end

function Elephant:CheckTableSize(index)
	local mLog
	local i=0
	
	mLog = Elephant.db.profile.maxlog
	
	while #Elephant.dbpc.char.logs[index].logs > mLog do
		table.remove(Elephant.dbpc.char.logs[index].logs, 1)
		i = i+1
	end
	
	return i
end

function Elephant:ChangeMaxLog(nb)
	if (nb < Elephant.defaultConf.minlogsize) then return end
	if (nb > Elephant.defaultConf.maxlogsize) then return end
	
	Elephant.db.profile.maxlog = nb
	
	local k
	for k in pairs(Elephant.dbpc.char.logs) do
		Elephant:CheckTableSize(k)
	end
	
	if (nb < Elephant.tempConf.currentline) then
		Elephant.tempConf.currentline = nb
	end
	
	Elephant:SetTitleInfoMaxLog()
	Elephant:ShowCurrentLog()
end

function Elephant:ChangeCombatMaxLog(nb)
	if (nb < Elephant.defaultConf.mincombatlogsize) then return end
	if (nb > Elephant.defaultConf.maxcombatlogsize) then return end
	
	Elephant.db.profile.maxcombatlog = nb
	
	Elephant:CheckTableSize(Elephant.defaultConf.defaultindexes.combat)
	
	if (nb < Elephant.tempConf.currentline) then
		Elephant.tempConf.currentline = nb
	end
	
	Elephant:SetTitleInfoMaxLog()
	Elephant:ShowCurrentLog()
end

function Elephant:EmptyCurrentLog()
	Elephant.dbpc.char.logs[Elephant.dbpc.char.currentlogindex].logs = {}
	if Elephant.dbpc.char.logs[Elephant.dbpc.char.currentlogindex].enabled then
		Elephant:AddHeaderToTable(Elephant.dbpc.char.logs[Elephant.dbpc.char.currentlogindex])
	end
	
	Elephant:Print( format(L['emptyconfirm'], Elephant.dbpc.char.logs[Elephant.dbpc.char.currentlogindex].name) )
	
	Elephant.tempConf.currentline = #Elephant.dbpc.char.logs[Elephant.dbpc.char.currentlogindex].logs
	Elephant:ShowCurrentLog()
end

function Elephant:ClearAllLogs()
	local k, v
	for k,v in pairs(Elephant.dbpc.char.logs) do
		v.logs = {}
		if v.enabled then
			Elephant:AddHeaderToTable(v)
		end
	end
	
	Elephant:Print(L['clearallconfirm'])
	
	Elephant.tempConf.currentline = #Elephant.dbpc.char.logs[Elephant.dbpc.char.currentlogindex].logs
	Elephant:ShowCurrentLog()
end

function Elephant:DeleteCurrentLog()
	Elephant:DeleteLog(Elephant.dbpc.char.currentlogindex)
end

function Elephant:DeleteLog(index)
	Elephant:Print( format(L['deleteconfirm'], Elephant.dbpc.char.logs[index].name) )
	Elephant.dbpc.char.logs[index] = nil
end

function Elephant:CaptureNewMessage(msg, index)
	table.insert(Elephant.dbpc.char.logs[index].logs, #Elephant.dbpc.char.logs[index].logs+1, msg)
	
	if not Elephant.dbpc.char.logs[index].hasMessage then
		Elephant.dbpc.char.logs[index].hasMessage = true
	end
	
	if Elephant.dbpc.char.currentlogindex == index then
		--[[ Moves the current line if it WAS at the last line ]]
		if Elephant.tempConf.currentline == (#Elephant.dbpc.char.logs[index].logs-1) then
			Elephant.tempConf.currentline = Elephant.tempConf.currentline + 1
		end
		
		Elephant.tempConf.currentline = Elephant.tempConf.currentline - Elephant:CheckTableSize(index)
		if Elephant.tempConf.currentline < 1 then
			Elephant.tempConf.currentline = 1
		end
		
		if Elephant.tempConf.currentline == #Elephant.dbpc.char.logs[index].logs then
			--[[ Adds the message to the screen ]]
			Elephant.gui.scrollingMessageFrame:AddMessage(Elephant:GetLiteralMessage(msg))
		end
		--[[ Updates current line text ]]
		Elephant:SetTitleInfoCurrentLine()
	else
		Elephant:CheckTableSize(index)
	end
end

function Elephant:Reset()
	Elephant:ResetSavedVariables()
	
	Elephant:ChatLogEnable(Elephant.db.profile.chatlog)
	Elephant:CombatLogEnable(Elephant.db.profile.combatlog)
	
	local k, v
	local lcname
	local found
	local channelName
	local i
	-- Max: 20 channels
	for i=1,20 do
		_,channelName = GetChannelName(i)
		
		if channelName ~= nil then
			lcname = string.lower(channelName)
			found = false
			for k,v in pairs(L['generalchats']) do
				if (lcname == k) or string.find(lcname, k .. " - ") then
					found = true
					break
				end
			end
			if not found then
				Elephant:InitCustomStructure(lcname, channelName)
				Elephant:CaptureNewMessage( { ['type'] = "SYSTEM", ['arg1'] = L['customchat']['join'] } , lcname)
			end
		end
	end
	
	Elephant:InitDefaultLogStructures()
	Elephant:InitCustomStructures()
	Elephant:AddHeaderToStructures(true)
	Elephant:ResetPosition()
	Elephant:ChangeLog(Elephant.dbpc.char.currentlogindex)
	
	Elephant:Print(L['resetconfirm'])
end

function Elephant:ResetSavedVariables()
	for k,v in pairs(Elephant:clone(Elephant.defaultConf.savedconfdefaults)) do
		Elephant.db.profile[k] = v
	end
	for k,v in pairs(Elephant:clone(Elephant.defaultConf.savedpercharconfdefaults)) do
		Elephant.dbpc.char[k] = v
	end
end

function Elephant:ToggleEnableCurrentLog()
	Elephant.dbpc.char.logs[Elephant.dbpc.char.currentlogindex].enabled = not Elephant.dbpc.char.logs[Elephant.dbpc.char.currentlogindex].enabled
	
	if Elephant.dbpc.char.logs[Elephant.dbpc.char.currentlogindex].enabled then
		Elephant:CaptureNewMessage( { ['arg1'] = " " } , Elephant.dbpc.char.currentlogindex)
		Elephant:CaptureNewMessage( { ['arg1'] = " " } , Elephant.dbpc.char.currentlogindex)
	end
	Elephant:CaptureNewMessage( { ['type'] = "SYSTEM", ['arg1'] = Elephant:GetEnableActionMsg(Elephant.dbpc.char.logs[Elephant.dbpc.char.currentlogindex].enabled) } , Elephant.dbpc.char.currentlogindex)
	
	Elephant:UpdateCurrentLogButtons()
end

function Elephant:AddFilter(arg1)
	local v
	for _,v in pairs(Elephant.db.profile.filters) do
		if v == arg1 then
			return
		end
	end
	
	table.insert(Elephant.db.profile.filters, arg1)
	Elephant:Print(format(L['filteradded'], arg1))
	
	local k,v
	for k,v in pairs(Elephant.dbpc.char.logs) do
		if type(k) ~= "number" and string.find(k, " ") == nil then
			if Elephant:IsFiltered(k) then
				Elephant:DeleteLog(k)
			end
		end
	end
end

function Elephant:IsFiltered(index)
	local v
	for _,v in pairs(Elephant.db.profile.filters) do
		v = "^" .. string.gsub(string.lower(v), "%*", "%.%*") .. "$"
		if string.match(index, v) ~= nil and not L['generalchats'][index] then
			return true
		end
	end
	
	return false
end

function Elephant:DeleteFilter(filterindex)
	if not Elephant.db.profile.filters[filterindex] then
		return
	else
		Elephant:Print(format(L['filterdeleted'], Elephant.db.profile.filters[filterindex]))
	end
	Elephant.db.profile.filters[filterindex] = nil
	
	local channelName
	local i
	-- Max: 20 channels
	for i=1,20 do
		_,channelName = GetChannelName(i)
		
		if
			channelName ~= nil and
			not Elephant.dbpc.char.logs[string.lower(channelName)] and
			not Elephant:IsFiltered(string.lower(channelName))
		then
			Elephant:InitCustomStructure(string.lower(channelName), channelName)
		end
	end
end

function Elephant:EnableCatcher(catch, channel)
	Elephant.db.profile.events[catch].channels[channel] = 1
end

function Elephant:DisableCatcher(catch, channel)
	Elephant.db.profile.events[catch].channels[channel] = 0
end